@@ -30,9 +30,10 @@ from uncompyle6.show import maybe_show_asm
 
 
 class ParserError(Exception):
-    def __init__(self, token, offset):
+    def __init__(self, token, offset, debug):
         self.token = token
         self.offset = offset
+        self.debug = debug
 
     def __str__(self):
         return "Parse error at or near `%r' instruction at offset %s\n" % (
@@ -206,9 +207,9 @@ class PythonParser(GenericASTBuilder):
                 else:
                     indent = "-> "
                 print("%s%s" % (indent, instructions[i]))
-            raise ParserError(err_token, err_token.offset)
+            raise ParserError(err_token, err_token.offset, self.debug["reduce"])
         else:
-            raise ParserError(None, -1)
+            raise ParserError(None, -1, self.debug["reduce"])
 
     def get_pos_kw(self, token):
         """Return then the number of positional parameters and
@@ -351,7 +352,7 @@ class PythonParser(GenericASTBuilder):
         stmt ::= try_except
         stmt ::= tryelsestmt
         stmt ::= tryfinallystmt
-        stmt ::= withstmt
+        stmt ::= with
         stmt ::= withasstmt
 
         stmt ::= del_stmt
