@@ -1,4 +1,4 @@
-#  Copyright (c) 2015-2018 by Rocky Bernstein
+#  Copyright (c) 2015-2019 by Rocky Bernstein
 #  Copyright (c) 2005 by Dan Pascu <dan@windowmaker.org>
 #  Copyright (c) 2000-2002 by hartmut Goebel <h.goebel@crazy-compilers.com>
 #
@@ -43,6 +43,10 @@ from xdis.bytecode import (
     _get_const_info)
 from xdis.util import code2num
 
+from uncompyle6 import PYTHON3
+if PYTHON3:
+    from sys import intern
+
 from uncompyle6.scanner import Scanner, Token
 
 class Scanner2(Scanner):
@@ -106,6 +110,17 @@ class Scanner2(Scanner):
 
 
 
+    @staticmethod
+    def extended_arg_val(arg):
+        """Return integer value of an EXTENDED_ARG operand.
+        In Python2 this always the operand value shifted 16 bits since
+        the operand is always 2 bytes. In Python 3.6+ this changes to one byte.
+        """
+        if PYTHON3:
+            return (arg << 16)
+        else:
+            return (arg << long(16))
+
     @staticmethod
     def unmangle_name(name, classname):
         """Remove __ from the end of _name_ if it starts with __classname__
@@ -272,6 +287,8 @@ class Scanner2(Scanner):
                             op_name = 'LOAD_DICTCOMP'
                         elif const.co_name == '<setcomp>':
                             op_name = 'LOAD_SETCOMP'
+                        else:
+                            op_name = "LOAD_CODE"
                         # verify() uses 'pattr' for comparison, since 'attr'
                         # now holds Code(const) and thus can not be used
                         # for comparison (todo: think about changing this)
@@ -871,9 +888,10 @@ class Scanner2(Scanner):
             # or a conditional assignment like:
             #   x = 1 if x else 2
             #
-            # There are other contexts we may need to consider
-            # like whether the target is "END_FINALLY"
-            # or if the condition jump is to a forward location
+            # There are other situations we may need to consider, like
+            # if the condition jump is to a forward location.
+            # Also the existence of a jump to the instruction after "END_FINALLY"
+            # will distinguish "try/else" from "try".
             code_pre_rtarget = code[pre_rtarget]
 
             if code_pre_rtarget in self.jump_forward:
@@ -1163,7 +1181,7 @@ class Scanner2(Scanner):
         Return a list with indexes to them or [] if none found.
         """
 
-        assert(start>=0 and end<=len(self.code) and start <= end)
+        assert(start >= 0 and end <= len(self.code) and start <= end)
 
         try:    None in instr
         except: instr = [instr]
