@@ -1,4 +1,4 @@
-#  Copyright (c) 2017, 2018 by Rocky Bernstein
+#  Copyright (c) 2017-2020 by Rocky Bernstein
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
 """Constants and initial table values used in pysource.py and fragments.py"""
 
 import re, sys
-from uncompyle6.parsers.astnode import AST
+from uncompyle6.parsers.treenode import SyntaxTree
 from uncompyle6 import PYTHON3
 from uncompyle6.scanners.tok import Token, NoneToken
 
@@ -27,46 +27,135 @@ else:
     maxint = sys.maxint
 
 
+# Operator precidence See
+# https://docs.python.org/2/reference/expressions.html#operator-precedence
+# or
+# https://docs.python.org/3/reference/expressions.html#operator-precedence
+# for a list. We keep the same top-to-botom order here as in the above links,
+# so we start with low precedence (high values) and go down in value.
+
+# Things at the bottom of this list below with high precedence (low value) will
+# tend to have parenthesis around them. Things at the top
+# of the list will tend not to have parenthesis around them.
+
+# Note: The values in this table are even numbers. Inside
+# various templates we use odd values. Avoiding equal-precedent comparisons
+# avoids ambiguity what to do when the precedence is equal.
+
+# The precidence of a key below applies the key, a node, and the its
+# *parent*. A node however sometimes sets the precidence for its
+# children. For example, "call" has precidence 2 so we don't get
+# additional the additional parenthesis of: ".. op (call())".  However
+# for call's children, it parameters, we set the the precidence high,
+# say to 100, to make sure we avoid additional prenthesis in
+# call((.. op ..)).
+
+PRECEDENCE = {
+    'yield':                 102,
+    'yield_from':            102,
+
+    '_mklambda':              30,
+
+    'if_exp':                 28, # If_Exp expression
+    'if_exp_lamdba':          28, # Lambda expression
+    'if_exp_not_lamdba':      28, # Lambda expression
+    'if_exp_not':             28,
+    'if_exp_true':            28,
+    'if_exp_ret':             28,
+
+    'or':                     26, # Boolean OR
+    'ret_or':                 26,
+
+    'and':                    24, # Boolean AND
+    'compare':                20, # in, not in, is, is not, <, <=, >, >=, !=, ==
+    'ret_and':                24,
+    'unary_not':              22, # Boolean NOT
+
+    'BINARY_AND':             14, # Bitwise AND
+    'BINARY_OR':              18, # Bitwise OR
+    'BINARY_XOR':             16, # Bitwise XOR
+
+    'BINARY_LSHIFT':          12, # Shifts <<
+    'BINARY_RSHIFT':          12, # Shifts >>
+
+    'BINARY_ADD':             10, # -
+    'BINARY_SUBTRACT':        10, # +
+
+    'BINARY_DIVIDE':          8,  # /
+    'BINARY_FLOOR_DIVIDE':    8,  # //
+    'BINARY_MATRIX_MULTIPLY': 8,  # @
+    'BINARY_MODULO':          8,  # Remainder, %
+    'BINARY_MULTIPLY':        8,  # *
+    'BINARY_TRUE_DIVIDE':     8,  # Division /
+
+    'unary_op':               6,  # +x, -x, ~x
+
+    'BINARY_POWER':           4,  # Exponentiation, *
+
+    'await_expr':             3,  # await x, *
+
+    'attribute':              2,  # x.attribute
+    'buildslice2':            2,  # x[index]
+    'buildslice3':            2,  # x[index:index]
+    'call':                   2,  # x(arguments...)
+    'delete_subscript':       2,
+    'slice0':                 2,
+    'slice1':                 2,
+    'slice2':                 2,
+    'slice3':                 2,
+    'store_subscript':        2,
+    'subscript':              2,
+    'subscript2':             2,
+
+    'dict':                   0,  # {expressions...}
+    'dict_comp':              0,
+    'generator_exp':          0,  # (expressions...)
+    'list':                   0,  # [expressions...]
+    'list_comp':              0,
+    'set_comp':               0,
+    'set_comp_expr':          0,
+    'unary_convert':          0,
+}
+
 LINE_LENGTH = 80
 
 # Some parse trees created below are used for comparing code
 # fragments (like 'return None' at the end of functions).
 
-RETURN_LOCALS = AST('return',
-                    [ AST('ret_expr', [AST('expr', [ Token('LOAD_LOCALS') ])]),
+RETURN_LOCALS = SyntaxTree('return',
+                    [ SyntaxTree('ret_expr', [SyntaxTree('expr', [ Token('LOAD_LOCALS') ])]),
                       Token('RETURN_VALUE')])
 
-NONE = AST('expr', [ NoneToken ] )
+NONE = SyntaxTree('expr', [ NoneToken ] )
 
-RETURN_NONE = AST('stmt',
-                  [ AST('return',
+RETURN_NONE = SyntaxTree('stmt',
+                  [ SyntaxTree('return',
                         [ NONE, Token('RETURN_VALUE')]) ])
 
-PASS = AST('stmts',
-           [ AST('sstmt',
-                 [ AST('stmt',
-                       [ AST('pass', [])])])])
+PASS = SyntaxTree('stmts',
+           [ SyntaxTree('sstmt',
+                 [ SyntaxTree('stmt',
+                       [ SyntaxTree('pass', [])])])])
 
-ASSIGN_DOC_STRING = lambda doc_string: \
-  AST('stmt',
-      [ AST('assign',
-            [ AST('expr', [ Token('LOAD_CONST', pattr=doc_string) ]),
-              AST('store', [ Token('STORE_NAME', pattr='__doc__')])
+ASSIGN_DOC_STRING = lambda doc_string, doc_load: \
+  SyntaxTree("stmt",
+      [ SyntaxTree("assign",
+            [ SyntaxTree("expr", [ Token(doc_load, pattr=doc_string, attr=doc_string) ]),
+              SyntaxTree("store", [ Token("STORE_NAME", pattr="__doc__")])
             ])])
 
-NAME_MODULE = AST('stmt',
-                [ AST('assign',
-                    [ AST('expr',
+NAME_MODULE = SyntaxTree('assign',
+                    [ SyntaxTree('expr',
                           [Token('LOAD_NAME', pattr='__name__', offset=0, has_arg=True)]),
-                      AST('store',
+                      SyntaxTree('store',
                           [ Token('STORE_NAME', pattr='__module__', offset=3, has_arg=True)])
-                      ])])
+                      ])
 
 # God intended \t, but Python has decided to use 4 spaces.
 # If you want real tabs, use Go.
-# TAB = '\t'
-TAB = ' ' * 4
-INDENT_PER_LEVEL = ' ' # additional intent per pretty-print level
+# TAB = "\t"
+TAB = " " * 4
+INDENT_PER_LEVEL = " " # additional intent per pretty-print level
 
 TABLE_R = {
     'STORE_ATTR':	( '%c.%[1]{pattr}', 0),
@@ -108,16 +197,20 @@ TABLE_DIRECT = {
     'INPLACE_AND':              ( '&=' ,),
     'INPLACE_OR':               ( '|=' ,),
     'INPLACE_XOR':              ( '^=' ,),
-    'binary_expr':              ( '%c %c %c', 0,
-                                (-1, 'binary_op'),
+
+    # bin_op (formerly "binary_expr") is the Python AST BinOp
+    'bin_op':                  ( '%c %c %c', 0,
+                                (-1, 'binary_operator'),
                                 ( 1, 'expr' ) ),
 
     'UNARY_POSITIVE':           ( '+',),
     'UNARY_NEGATIVE':           ( '-',),
     'UNARY_INVERT':             ( '~'),
-    'unary_expr':               ( '%c%c',
-                                  (1, 'unary_op'),
-                                  (0, 'expr') ),
+
+    # unary_op (formerly "unary_expr") is the Python AST UnaryOp
+    'unary_op':                 ( '%c%c',
+                                 (1, 'unary_operator'),
+                                 (0, 'expr') ),
 
     'unary_not':	            ( 'not %c',
                                   (0, 'expr' ) ),
@@ -138,10 +231,12 @@ TABLE_DIRECT = {
                                   (1, 100), (2, 100) ),
 
     'IMPORT_FROM':              ( '%{pattr}', ),
+    'IMPORT_NAME_ATTR':         ( '%{pattr}', ),
     'attribute':	        ( '%c.%[1]{pattr}',
-                                (0, 'expr')),
-    'LOAD_FAST':	            ( '%{pattr}', ),
-    'LOAD_NAME':	            ( '%{pattr}', ),
+                                  (0, 'expr')),
+    'LOAD_STR':	                ( '%{pattr}', ),
+    'LOAD_FAST':	        ( '%{pattr}', ),
+    'LOAD_NAME':	        ( '%{pattr}', ),
     'LOAD_CLASSNAME':	        ( '%{pattr}', ),
     'LOAD_GLOBAL':	        ( '%{pattr}', ),
     'LOAD_DEREF':	        ( '%{pattr}', ),
@@ -150,14 +245,17 @@ TABLE_DIRECT = {
     'DELETE_FAST':	        ( '%|del %{pattr}\n', ),
     'DELETE_NAME':	        ( '%|del %{pattr}\n', ),
     'DELETE_GLOBAL':	        ( '%|del %{pattr}\n', ),
-    'delete_subscr':            ( '%|del %c[%c]\n', 0, 1,),
-    'subscript':                ( '%c[%p]',
-                                      (0, 'expr'),
-                                      (1, 100) ),
-    'subscript2':               ( '%c[%c]',
-                                      (0, 'expr'),
+    'delete_subscript':         ( '%|del %p[%c]\n',
+                                  (0, 'expr', PRECEDENCE['subscript']), (1, 'expr') ),
+    'subscript':                ( '%p[%c]',
+                                      (0, 'expr', PRECEDENCE['subscript']),
                                       (1, 'expr') ),
-    'store_subscr':	        ( '%c[%c]', 0, 1),
+    'subscript2':               ( '%p[%c]',
+                                      (0, 'expr', PRECEDENCE['subscript']),
+                                      (1, 'expr') ),
+    'store_subscript':	        ( '%p[%c]',
+                                  (0, 'expr', PRECEDENCE['subscript']),
+                                  (1, 'expr') ),
     'STORE_FAST':	        ( '%{pattr}', ),
     'STORE_NAME':	        ( '%{pattr}', ),
     'STORE_GLOBAL':	        ( '%{pattr}', ),
@@ -171,6 +269,10 @@ TABLE_DIRECT = {
     'attribute_w_parens':	( '(%c).%[1]{pattr}',
                                   (0, 'expr')),
 
+    # This nonterminal we create on the fly in semantic routines
+    'store_w_parens':	( '(%c).%[1]{pattr}',
+                                  (0, 'expr')),
+
     'unpack_list':	        ( '[%C]',
                                       (1, maxint, ', ') ),
     'build_tuple2':	        ( '%P',
@@ -178,17 +280,21 @@ TABLE_DIRECT = {
 
     'list_iter':	    ( '%c', 0 ),
     'list_for':		    ( ' for %c in %c%c', 2, 0, 3 ),
-    'list_if':		    ( ' if %c%c', 0, 2 ),
-    'list_if_not':		( ' if not %p%c', (0, 22), 2 ),
+    'list_if':		    ( ' if %p%c',
+                              (0, 'expr', 27), 2 ),
+    'list_if_not':	    ( ' if not %p%c',
+                              (0, 'expr', PRECEDENCE['unary_not']),
+                              2 ),
     'lc_body':		    ( '', ),	# ignore when recursing
 
     'comp_iter':	    ( '%c', 0 ),
     'comp_if':		    ( ' if %c%c', 0, 2 ),
-    'comp_if_not':	    ( ' if not %p%c', (0, 22), 2 ),
+    'comp_if_not':	    ( ' if not %p%c',
+                              (0, 'expr', PRECEDENCE['unary_not']), 2 ),
     'comp_body':	    ( '', ),	# ignore when recusing
-    'set_comp_body':    ( '%c', 0 ),
-    'gen_comp_body':    ( '%c', 0 ),
-    'dict_comp_body':   ( '%c:%c', 1, 0 ),
+    'set_comp_body':        ( '%c', 0 ),
+    'gen_comp_body':        ( '%c', 0 ),
+    'dict_comp_body':       ( '%c:%c', 1, 0 ),
 
     'assign':		    ( '%|%c = %p\n', -1, (0, 200) ),
 
@@ -197,37 +303,47 @@ TABLE_DIRECT = {
     # which we don't use here.
     'aug_assign1':      ( '%|%c %c %c\n', 0, 2, 1),
 
-    'aug_assign2':	    ( '%|%c.%[2]{pattr} %c %c\n', 0, -3, -4 ),
-    'designList':	    ( '%c = %c', 0, -1 ),
+    'aug_assign2':      ( '%|%c.%[2]{pattr} %c %c\n', 0, -3, -4 ),
+    'designList':       ( '%c = %c', 0, -1 ),
     'and':          	( '%c and %c', 0, 2 ),
     'ret_and':        	( '%c and %c', 0, 2 ),
     'and2':          	( '%c', 3 ),
     'or':           	( '%c or %c', 0, 2 ),
     'ret_or':           ( '%c or %c', 0, 2 ),
-    'conditional':      ( '%p if %p else %p', (2, 27), (0, 27), (4, 27) ),
-    'conditional_true': ( '%p if 1 else %p', (0, 27), (2, 27) ),
-    'ret_cond':         ( '%p if %p else %p', (2, 27), (0, 27), (-1, 27) ),
-    'conditional_not':  ( '%p if not %p else %p', (2, 27), (0, 22), (4, 27) ),
-    'ret_cond_not':     ( '%p if not %p else %p', (2, 27), (0, 22), (-1, 27) ),
-    'conditional_lambda':
-                        ( '%c if %c else %c',
-                          (2, 'expr'), 0, 4 ),
-    'conditional_not_lambda':
-                        ( '%c if not %c else %c',
-                          (2, 'expr'), 0, 4 ),
+    'if_exp':           ( '%p if %c else %c',
+                          (2, 'expr', 27), 0, 4 ),
+    'if_exp_lambda':    ( '%p if %c else %c',
+                          (2, 'expr', 27), (0, 'expr'), 4 ),
+    'if_exp_true':      ( '%p if 1 else %c', (0, 'expr', 27), 2 ),
+    'if_exp_ret':       ( '%p if %p else %p', (2, 27), (0, 27), (-1, 27) ),
+    'if_exp_not':       ( '%p if not %p else %p',
+                          (2, 27),
+                          (0, "expr", PRECEDENCE['unary_not']),
+                          (4, 27) ),
+    'if_exp_not_lambda':
+                        ( '%p if not %c else %c',
+                          (2, 'expr', 27), 0, 4 ),
 
     'compare_single':	    ( '%p %[-1]{pattr.replace("-", " ")} %p', (0, 19), (1, 19) ),
     'compare_chained':	    ( '%p %p', (0, 29), (1, 30)),
     'compare_chained1':	    ( '%[3]{pattr.replace("-", " ")} %p %p', (0, 19), (-2, 19)),
     'compare_chained2':	    ( '%[1]{pattr.replace("-", " ")} %p', (0, 19)),
 #   'classdef': 	(), # handled by n_classdef()
+
+    # A custom rule in n_function def distinguishes whether to call this or
+    # function_def_async
     'function_def':         ( '\n\n%|def %c\n', -2), # -2 to handle closures
+
     'function_def_deco':    ( '\n\n%c', 0),
     'mkfuncdeco':  	    ( '%|@%c\n%c', 0, 1),
+
+    # A custom rule in n_function def distinguishes whether to call this or
+    # function_def_async
     'mkfuncdeco0':  	    ( '%|def %c\n', 0),
+
     'classdefdeco':  	    ( '\n\n%c', 0),
     'classdefdeco1':  	    ( '%|@%c\n%c', 0, 1),
-    'kwarg':    	    ( '%[0]{pattr}=%c', 1),
+    'kwarg':    	    ( '%[0]{pattr}=%c', 1),  # Change when Python 2 does LOAD_STR
     'kwargs':    	    ( '%D', (0, maxint, ', ') ),
     'kwargs1':    	    ( '%D', (0, maxint, ', ') ),
 
@@ -242,6 +358,13 @@ TABLE_DIRECT = {
     'print_nl_to':	    ( '%|print >> %c\n', 0 ),
     'print_to_items':	( '%C', (0, 2, ', ') ),
 
+    # This is only generated by transform
+    # it is a string at the beginning of a function that is *not* a docstring
+    # 3.7 test_fstring.py tests for this kind of crap.
+    # For compatibility with older Python, we'll use "%" instead of
+    # a format string.
+    "string_at_beginning":   ( '%|"%%s" %% %c\n', 0),
+
     'call_stmt':	    ( '%|%p\n', (0, 200)),
     'break':	            ( '%|break\n', ),
     'continue':	            ( '%|continue\n', ),
@@ -253,15 +376,29 @@ TABLE_DIRECT = {
 #    'return':	        ( '%|return %c\n', 0),
     'return_if_stmt':	( 'return %c\n', 0),
 
-    'ifstmt':		    ( '%|if %c:\n%+%c%-', 0, 1 ),
-    'iflaststmt':		( '%|if %c:\n%+%c%-', 0, 1 ),
-    'iflaststmtl':		( '%|if %c:\n%+%c%-', 0, 1 ),
-    'testtrue':         ( 'not %p', (0, 22) ),
+    "ifstmt":		( "%|if %c:\n%+%c%-",
+                            0, # "testexpr" or "testexpr_then"
+                            1, # "_ifstmts_jump" or "return_stmts"
+                        ),
+    'iflaststmt':	( "%|if %c:\n%+%c%-", 0, 1 ),
+    'iflaststmtl':	( "%|if %c:\n%+%c%-", 0, 1 ),
+    'testtrue':         ( "not %p",
+                          (0, PRECEDENCE['unary_not']) ),
 
+    # Generally the args here are 0: (some sort of) "testexpr",
+    #                             1: (some sort of) "cstmts_opt",
+    #                             2 or 3: "else_suite"
+    # But unfortunately there are irregularities, For example, 2.6- uses "testexpr_then"
+    # and sometimes "cstmts" instead of "cstmts_opt" happens.
+    # Down the line we might isolate these into version-specific rules.
     'ifelsestmt':	( '%|if %c:\n%+%c%-%|else:\n%+%c%-', 0, 1, 3 ),
     'ifelsestmtc':	( '%|if %c:\n%+%c%-%|else:\n%+%c%-', 0, 1, 3 ),
     'ifelsestmtl':	( '%|if %c:\n%+%c%-%|else:\n%+%c%-', 0, 1, 3 ),
-    'ifelifstmt':	( '%|if %c:\n%+%c%-%c', 0, 1, 3 ),
+
+    #  These are created only via transformation
+    'ifelifstmt':	( '%|if %c:\n%+%c%-%c',
+                          0, # "testexpr" or "testexpr_then"
+                          1, 3 ),
     'elifelifstmt':	( '%|elif %c:\n%+%c%-%c', 0, 1, 3 ),
     'elifstmt':		( '%|elif %c:\n%+%c%-', 0, 1 ),
     'elifelsestmt':	( '%|elif %c:\n%+%c%-%|else:\n%+%c%-', 0, 1, 3 ),
@@ -275,26 +412,47 @@ TABLE_DIRECT = {
     'while1stmt':	    ( '%|while 1:\n%+%c%-\n\n', 1 ),
     'while1elsestmt':   ( '%|while 1:\n%+%c%-%|else:\n%+%c%-\n\n', 1, -2 ),
     'whileelsestmt':	( '%|while %c:\n%+%c%-%|else:\n%+%c%-\n\n', 1, 2, -2 ),
+    'whileelsestmt2':	( '%|while %c:\n%+%c%-%|else:\n%+%c%-\n\n', 1, 2, -3 ),
     'whileelselaststmt':	( '%|while %c:\n%+%c%-%|else:\n%+%c%-', 1, 2, -2 ),
-    'for':              ( '%|for %c in %c:\n%+%c%-\n\n', (3, 'store'), 1, 4 ),
+
+    # Note: Python 3.8+ changes this
+    'for':              ( '%|for %c in %c:\n%+%c%-\n\n',
+                          (3, 'store'),
+                          (1, 'expr'),
+                          (4, 'for_block') ),
     'forelsestmt':	    (
-        '%|for %c in %c:\n%+%c%-%|else:\n%+%c%-\n\n', (3, 'store'), 1, 4, -2 ),
+        '%|for %c in %c:\n%+%c%-%|else:\n%+%c%-\n\n',
+                          (3, 'store'),
+                          (1, 'expr'),
+                          (4, 'for_block'), -2 ),
     'forelselaststmt':	(
-        '%|for %c in %c:\n%+%c%-%|else:\n%+%c%-', (3, 'store'), 1, 4, -2 ),
+        '%|for %c in %c:\n%+%c%-%|else:\n%+%c%-',
+                          (3, 'store'),
+                          (1, 'expr'),
+                          (4, 'for_block'), -2 ),
     'forelselaststmtl':	(
-        '%|for %c in %c:\n%+%c%-%|else:\n%+%c%-\n\n', (3, 'store'), 1, 4, -2 ),
+        '%|for %c in %c:\n%+%c%-%|else:\n%+%c%-\n\n',
+                          (3, 'store'),
+                          (1, 'expr'),
+                          (4, 'for_block'), -2 ),
+
     'try_except':       ( '%|try:\n%+%c%-%c\n\n', 1, 3 ),
     'tryelsestmt':	    ( '%|try:\n%+%c%-%c%|else:\n%+%c%-\n\n', 1, 3, 4 ),
     'tryelsestmtc':	    ( '%|try:\n%+%c%-%c%|else:\n%+%c%-', 1, 3, 4 ),
     'tryelsestmtl':	    ( '%|try:\n%+%c%-%c%|else:\n%+%c%-', 1, 3, 4 ),
+
+    # Note: this is generated generated by grammar rules but in this phase.
     'tf_try_except':	( '%c%-%c%+', 1, 3 ),
     'tf_tryelsestmt':	( '%c%-%c%|else:\n%+%c', 1, 3, 4 ),
+
     'tryfinallystmt':	( '%|try:\n%+%c%-%|finally:\n%+%c%-\n\n', 1, 5 ),
     'except':           ( '%|except:\n%+%c%-', 3 ),
     'except_cond1':	    ( '%|except %c:\n', 1 ),
+    'except_cond2':     ( '%|except %c as %c:\n',
+                          (1, 'expr'), (5, 'store') ),
     'except_suite':     ( '%+%c%-%C', 0, (1, maxint, '') ),
 
-    # In Python 3.6, this is more complicated in the presence of "returns"
+    # In Python 3.6+, this is more complicated in the presence of "returns"
     'except_suite_finalize':     ( '%+%c%-%C', 1, (3, maxint, '') ),
 
     'pass':	            ( '%|pass\n', ),
@@ -314,89 +472,19 @@ MAP_R0 = (TABLE_R0, -1, 0)
 MAP_R = (TABLE_R, -1)
 
 MAP = {
-    'stmt':		MAP_R,
-    'call':	        MAP_R,
-    'del_stmt':		MAP_R,
-    'store':	        MAP_R,
-    'exprlist':		MAP_R0,
-}
-
-# Operator precidence
-# See https://docs.python.org/2/reference/expressions.html
-# or https://docs.python.org/3/reference/expressions.html
-# for a list.
-
-# Things at the top of this list below with low-value precidence will
-# tend to have parenthesis around them. Things at the bottom
-# of the list will tend not to have parenthesis around them.
-PRECEDENCE = {
-    'list':                   0,
-    'dict':                   0,
-    'unary_convert':          0,
-    'dict_comp':              0,
-    'set_comp':               0,
-    'set_comp_expr':          0,
-    'list_comp':              0,
-    'generator_exp':          0,
-
-    'attribute':              2,
-    'subscript':              2,
-    'subscript2':             2,
-    'slice0':                 2,
-    'slice1':                 2,
-    'slice2':                 2,
-    'slice3':                 2,
-    'buildslice2':            2,
-    'buildslice3':            2,
-    'call':                   2,
-
-    'BINARY_POWER':           4,
-
-    'unary_expr':             6,
-
-    'BINARY_MULTIPLY':        8,
-    'BINARY_DIVIDE':          8,
-    'BINARY_TRUE_DIVIDE':     8,
-    'BINARY_FLOOR_DIVIDE':    8,
-    'BINARY_MODULO':          8,
-
-    'BINARY_ADD':             10,
-    'BINARY_SUBTRACT':        10,
-
-    'BINARY_LSHIFT':          12,
-    'BINARY_RSHIFT':          12,
-
-    'BINARY_AND':             14,
-    'BINARY_XOR':             16,
-    'BINARY_OR':              18,
-
-    'compare':                20,
-    'unary_not':              22,
-    'and':                    24,
-    'ret_and':                24,
-
-    'or':                     26,
-    'ret_or':                 26,
-
-    'conditional':            28,
-    'conditional_lamdba':     28,
-    'conditional_not_lamdba': 28,
-    'conditionalnot':         28,
-    'ret_cond':               28,
-    'ret_cond_not':           28,
-
-    '_mklambda':              30,
-
-    'yield':                 101,
-    'yield_from':            101
+    "stmt":		MAP_R,
+    "call":	        MAP_R,
+    "del_stmt":		MAP_R,
+    "store":	        MAP_R,
+    "exprlist":		MAP_R0,
 }
 
 ASSIGN_TUPLE_PARAM = lambda param_name: \
-             AST('expr', [ Token('LOAD_FAST', pattr=param_name) ])
+             SyntaxTree("expr", [ Token("LOAD_FAST", pattr=param_name) ])
 
-escape = re.compile(r'''
+escape = re.compile(r"""
             (?P<prefix> [^%]* )
             % ( \[ (?P<child> -? \d+ ) \] )?
                 ((?P<type> [^{] ) |
                  ( [{] (?P<expr> [^}]* ) [}] ))
-        ''', re.VERBOSE)
+        """, re.VERBOSE)
